#pragma once

#include <cppmariadb/config.h>

namespace cppmariadb {
namespace __impl {

    /**
     * @brief Wrap any mariadb library handle.
     *
     * @tparam T    Handle to wrap.
     */
    template<class T>
    struct handle
    {
    private:
        T _handle;  //!< Handle of mariadb library.

    public:
        /**
         * @brief Value constructor. Initialize the object with the given mariadb handle.
         */
        inline handle(T h);

        /**
         * @brief Move constructor. Move the handle stored in the other object to this object.
         */
        inline handle(handle&& other);

    public:
        /**
         * @brief Operator to implicit convert the handle object to its underlying mariadb handle.
         */
        inline operator T() const;

        /**
         * @brief Get the handle stored in this object.
         */
        inline const T& get_handle() const;

    protected:
        /**
         * @brief Set the handle.
         */
        inline void set_handle(T h);
    };

} }
